module FlirtChecker

  module Pheno
    refine String do
      def to_phenotype
        self[-1] == "O" ? self[0] : self.squeeze
      end
    end
  end

  class Blood
    using Pheno

    BLOOD_GROUP = {
      a: ["A","A","A","O"],
      b: ["B","B","B","O"],
      o: ["O","O"],
      ab: ["A","B"]
    }

    attr_accessor :male, :female, :genotype

    def initialize(male, female, genotype: false)
      @male = male
      @female = female
      @genotype = genotype
    end

    def male
      @male.downcase.to_sym
    end

    def female
      @female.downcase.to_sym
    end

    def combo
      combo_with_rate.keys
    end

    def combo_with_rate
      c = combination
      hash = c.reduce(Hash.new(0)){ |h,v| h[@genotype ? v : v.to_phenotype] += 1; h }
      hash.each_with_object({}) { |(k,v), h| h[k] = v.to_f / c.size }
    end

  private

    def combination
      #parameterize
      valid_blood_type?
      BLOOD_GROUP[male].product(BLOOD_GROUP[female]).map { |e| e.sort.join }
    end

    def valid_blood_type?
      raise ArgumentError, "Invalid Blood Type" unless BLOOD_GROUP.key?(male) && BLOOD_GROUP.key?(female)
    end
  end
end
